#include "cutter.h"

#include <QDebug>

void Cutter::_clear() {
	saturated = false;
	started = false;
	silence = 0;
	lastSilenceWaiting = NULL;
	firstSilenceEnding = NULL;
	channel.clear();
	channel.setSamplerate(getSamplerate());
}

void Cutter::_normalize() {
	int maxval = channel.maxval();
	if (maxval < 0) return;

	double level = (double) channel.absmax() / (double) maxval;
	if (level < 0.9 * 0.9) channel.mul(0.9 / level);
}

void Cutter::clear() {
	mutex.lock();
	_clear();
	mutex.unlock();
}

bool Cutter::begin() {
	clear();
	return Recorder::begin();
}

bool Cutter::end() {
	clear();
	return Recorder::end();
}

bool Cutter::_isValid() {
	double from = _recordFrom();
	if (from < marginBefore)
		return false;

	if (_recordTo() - from < minimalLength)
		return false;

	return !saturated;
}

double Cutter::_recordFrom() {
	return lastSilenceWaiting == NULL ? 0 : channel.posT(lastSilenceWaiting) + channel.duration(lastSilenceWaiting);
}

double Cutter::_recordTo() {
	return firstSilenceEnding == NULL ? 0 : channel.posT(firstSilenceEnding);
}

void Cutter::_addSamplesWaiting(Samples *samples, double level) {
	if (level < thresholdSilence)
		lastSilenceWaiting = samples;

	double duration = channel.duration();
	if (level > thresholdStart) {
		started = true;
		return;
	}

	if (duration > bufferLength) {
		channel.ltrim(duration - bufferLength);
		if (channel.pos(lastSilenceWaiting) == (size_t) -1)
			lastSilenceWaiting = NULL;
	}
}

void Cutter::_addSamplesEnding(Samples *samples, double level) {
	if (level < thresholdSilence) {
		if (!firstSilenceEnding) firstSilenceEnding = samples;
		silence += channel.duration(samples);
	}
	else {
		firstSilenceEnding = NULL;
		silence = 0;
	}

	if (silence > stopSilence) {
		if (_isValid()) {
			channel.ltrim(_recordFrom() - marginBefore);
			channel.rtrim(stopSilence - marginAfter);
			if (normalize) _normalize();
			_save();
			terminated = true;
		}
		_clear();
	}
}

void Cutter::_addSamples(Samples *samples) {
	channel.push_back(samples);

	double level = (double) samples->absmax() / (double) samples->maxval();
	if (level > thresholdSaturation) {
		saturated = true;
	}

	if (!started) {
		_addSamplesWaiting(samples, level);
	}
	else {
		_addSamplesEnding(samples, level);
	}
}

void Cutter::addSamples(Samples *samples) {
	mutex.lock();
	terminated = false;
	_addSamples(samples);
	bool _terminated = terminated;
	mutex.unlock();

	if (_terminated) {
		terminate();
	}
}

double Cutter::getThresholdStart() const {
	return thresholdStart;
}

void Cutter::setThresholdStart(double value) {
	thresholdStart = value;
}

double Cutter::getThresholdSilence() const {
	return thresholdSilence;
}

void Cutter::setThresholdSilence(double value) {
	thresholdSilence = value;
}

double Cutter::getThresholdSaturation() const {
	return thresholdSaturation;
}

void Cutter::setThresholdSaturation(double value) {
	thresholdSaturation = value;
}

double Cutter::getMarginBefore() const {
	return marginBefore;
}

void Cutter::setMarginBefore(double value) {
	marginBefore = value;
}

double Cutter::getMarginAfter() const {
	return marginAfter;
}

void Cutter::setMarginAfter(double value) {
	marginAfter = value;
}

double Cutter::getStopSilence() const {
	return stopSilence;
}

void Cutter::setStopSilence(double value) {
	stopSilence = value;
}

bool Cutter::getNormalize() const {
	return normalize;
}

void Cutter::setNormalize(bool value) {
	normalize = value;
}

Cutter::Cutter() : started(false), saturated(false), silence(0), lastSilenceWaiting(NULL), firstSilenceEnding(NULL), thresholdStart(0.15), thresholdSilence(0.04), thresholdSaturation(0.95), bufferLength(0.8), marginBefore(0.25), marginAfter(0.25), stopSilence(0.3), minimalLength(0.20), normalize(true) {
}

Cutter::~Cutter() {
	clear();
}
