#include "cutter_widget.h"

#include <QDebug>
#include <QPainter>
#include <QApplication>
#include <QPaintEvent>
#include <QSettings>
#include <cmath>

#include "codec_flac.h"

CutterWidget::CutterWidget(QWidget *parent) : QFrame(parent), factor(0), factorT(160.0) {
	setFrameStyle(QFrame::Panel | QFrame::Sunken);
	connect(this, SIGNAL(updated()), this, SLOT(refresh()));

	//Load settings
	QSettings settings;
	setThresholdStart(settings.value("ThresholdStart", 0.15).toDouble());
	setThresholdSilence(settings.value("ThresholdSilence", 0.05).toDouble());
	setThresholdSaturation(settings.value("ThresholdSaturation", 0.95).toDouble());
	setMarginAfter(settings.value("MarginAfter", 0.25).toDouble());
	setMarginBefore(settings.value("MarginBefore", 0.25).toDouble());
	setStopSilence(settings.value("StopSilence", 0.30).toDouble());
	setNormalize(settings.value("Normalize", true).toBool());
}

CutterWidget::~CutterWidget() {
	//Save settings
	QSettings settings;
	settings.setValue("ThresholdStart", getThresholdStart());
	settings.setValue("ThresholdSilence", getThresholdSilence());
	settings.setValue("ThresholdSaturation", getThresholdSaturation());
	settings.setValue("MarginAfter", getMarginAfter());
	settings.setValue("MarginBefore", getMarginBefore());
	settings.setValue("StopSilence", getStopSilence());
	settings.setValue("Normalize", getNormalize());
}

void CutterWidget::addSamples(Samples *samples) {
	Cutter::addSamples(samples);
	emit updated();
}

void CutterWidget::refresh() {
	update();
}

bool CutterWidget::begin() {
	bool result = Cutter::begin();
	emit changeState();
	return result;
}

bool CutterWidget::end() {
	bool result = Cutter::end();
	emit changeState();
	return result;
}

void CutterWidget::_save() {
	EncoderFlac encoder;
	if (!encoder.init(channel)) {
		qDebug() << "could not init encoder"; 
		return;
	}

	if (!encoder.setTag(tag)) {
		qDebug() << "could not set tag"; 
		return;
	}

	if (!encoder.open(target.toLocal8Bit())) {
		qDebug() << "could not open file"; 
		return;
	}
 
	if (!encoder.encode(channel)) {
		qDebug() << "could not encode channel"; 
		return;
	}
}

void CutterWidget::terminate() {
	emit terminated();
}

void CutterWidget::setTarget(QString value) {
	target = value;
}

int CutterWidget::height() const {
	return 48 << factor;
}

QSize CutterWidget::sizeHint() const {
	return QSize(1280, height());
}

void CutterWidget::mousePressEvent(QMouseEvent *) {
	factor = (factor + 1) % 4;
	setMinimumSize(QSize(40, height()));
}

void CutterWidget::paintThreshold(QPainter &painter, int h, int w) {
	painter.drawLine(0, h, w, h);
}

void CutterWidget::paintArea(QPainter &painter, double t, double duration, int h, QColor color) {
	painter.setBrush(color);
	painter.setPen(color);
	painter.drawRect(t * factorT, 0, duration * factorT, h);
}

void CutterWidget::paintEvent(QPaintEvent *ev) {
	QFrame::paintEvent(ev);

	QPainter painter(this);

	//draw bars
	int h = height() - 3;
	int w = width() - 3;
	painter.translate(1, h + 1);
	painter.scale(1, -1);
  
	mutex.lock();

	//Areas
	if (started) {
		double from = _recordFrom();
		paintArea(painter, from - marginBefore, marginBefore, h, QColor(220, 220, 220));
		if (silence == 0) {
			paintArea(painter, from, channel.duration() - from, h, QColor(240, 240, 240));
		}
		else {
			double to = _recordTo();
			paintArea(painter, from, to - from, h, QColor(240, 240, 240));
			paintArea(painter, to, marginAfter, h, QColor(220, 220, 220));
		}
	}

	//Threshold
	painter.setPen(started ? QColor(0, 255, 0) : QColor(0, 128, 0));
	paintThreshold(painter, h * thresholdStart, w);

	painter.setPen(silence > 0 ? QColor(0, 255, 0) : QColor(0, 128, 0));
	paintThreshold(painter, h * thresholdSilence, w);

	painter.setPen(saturated ? QColor(255, 0, 0) : QColor(128, 0, 0));
	paintThreshold(painter, h * thresholdSaturation, w);

	//Samples
	painter.setPen(QColor(0, 0, 0));
	painter.setBrush(QColor(128, 128, 128));
	double t = 0;
	for (Channel::iterator it = channel.begin(); it != channel.end(); it++) {
		Samples *samples = *it;
		double d = channel.duration(samples);
		int ly = samples->absmax() * h / samples->maxval();
		painter.drawRect(round(t * factorT), 0, round(d * factorT) - 2, ly);
		t += d;
	}

	mutex.unlock();
}

void CutterWidget::setThresholdStart(double value) {
	Cutter::setThresholdStart(value);
	emit updated();
}

void CutterWidget::setThresholdSilence(double value) {
	Cutter::setThresholdSilence(value);
	emit updated();
}

void CutterWidget::setThresholdSaturation(double value) {
	Cutter::setThresholdSaturation(value);
	emit updated();
}

