#include "recorder_item.h"

#include <QPainter>
#include <QFontMetrics>
#include <QKeyEvent>
#include <QStringList>

#include "recorder_view.h"

RecorderItemData::RecorderItemData(RecorderItemData const *value) {
	_text = value->text();
	_filename = value->filename();
	_fields = value->fields();
}

RecorderItemData::RecorderItemData(QString text, QString filename, QString fields) : _text(text), _filename(filename), _fields(fields) {
}

QString RecorderItemData::text() const {
	return _text;
}

void RecorderItemData::setText(QString const value) {
	_text = value;
}

QString RecorderItemData::filename() const {
	return _filename;
}

void RecorderItemData::setFilename(QString const value) {
	_filename = value;
}

QString RecorderItemData::fields() const {
	return _fields;
}

void RecorderItemData::setFields(QString const value) {
	_fields = value;
}

void RecorderItemData::toTag(Tag &tag) const {
	tag.add("SWAC_TEXT", _text);

	if (_fields.isNull()) return;

	QStringList vectors = _fields.split(";");
	for (QStringList::const_iterator it = vectors.begin(); it != vectors.end(); it++) {
		QStringList kv = it->split("=");
		if (kv.count() == 2) {
			tag.add(kv[0], kv[1]);
		}
	}
}

RecorderItem::RecorderItem(RecorderWidget *_parent, RecorderItemEditor *_editor, RecorderItemData const *_data) : QWidget(_parent), RecorderItemData(_data), widget(_parent), editor(_editor), selected(false) {
	setVisible(true);
}

RecorderItem::RecorderItem(RecorderWidget *_parent, RecorderItemEditor *_editor, QString const text) : QWidget(_parent), RecorderItemData(text), widget(_parent), editor(_editor), selected(false) {
	setVisible(true);
}

QSize RecorderItem::sizeHint() const {
	QFontMetrics fm(font());
	QSize innerSize = _text.isNull()
		? fm.size(Qt::TextSingleLine, "") + QSize(100, 0)
		: fm.size(Qt::TextSingleLine, _text);
	return innerSize + QSize(10, 10);
}

void RecorderItem::setText(QString const value) {
	RecorderItemData::setText(value);
	widget->doLayout();
}

void RecorderItem::setSelected(bool value) {
	selected = value;
	update();
	if (selected) setFocus(Qt::PopupFocusReason);
}

void RecorderItem::mousePressEvent(QMouseEvent *) {
	widget->setSelected(this);
}

bool RecorderItem::sndExists() {
	return widget->sndExists(this);
}

void RecorderItem::paintEvent(QPaintEvent *) {
	QPainter painter(this);

	//background
	QColor bg = (sndExists()) ? QColor(230, 230, 230) : QColor(255, 255, 255);
	if (selected) bg = (widget->getRecording()) ? QColor(255, 200, 200) : QColor(200, 200, 255);
	painter.setBrush(bg);

	painter.setPen(selected ? QColor(0, 0, 0) : bg);
	painter.drawRect(rect().adjusted(0, 0, -1, -1));

	painter.setPen(QColor(0, 0, 0));
	painter.drawText(rect().adjusted(5, 5, -5, -5), _text);
}

void RecorderItem::edit() {
	editor->activate(this);
}

//---------Editor
RecorderItemEditor::RecorderItemEditor() {
	setFrame(false);
	setVisible(false);
}

void RecorderItemEditor::activate(RecorderItem *_item) {
	item = _item;
	setParent(item);
	setText(item->text());
	QSize is = item->sizeHint();
	setGeometry(QRect(0, 0, is.width(), is.height()));
	setVisible(true);
	setFocus(Qt::PopupFocusReason);
}

void RecorderItemEditor::hide() {
	item->setFocus(Qt::PopupFocusReason);
	item = NULL;
	setParent(NULL);
	setText("");
	setVisible(false);
}

void RecorderItemEditor::keyPressEvent(QKeyEvent *event) {
        switch (event->key()) {
	case Qt::Key_Escape:
		hide();
		break;
	case Qt::Key_Return:
		item->setText(text());
		hide();
		break;
	default:
		QLineEdit::keyPressEvent(event);
	}
}

