#include "recorder_view.h"

#include <QDebug>
#include <QKeyEvent>
#include <QScrollBar>
#include <QFile>

RecorderWidget::RecorderWidget(RecorderView *_view, SndExistsCb *_sndExistsCb, void *_sndExistsCbData) : QWidget(_view), view(_view), selected(-1), recording(false), sndExistsCb(_sndExistsCb), sndExistsCbData(_sndExistsCbData) {
	editor = new RecorderItemEditor;
	setFocusPolicy(Qt::StrongFocus);
}

void RecorderWidget::setRecording(bool value) {
	recording = value;
	RecorderItem *selected = itemSelected();
	if (selected) selected->update();
}

bool RecorderWidget::getRecording() const {
	return recording;
}

bool RecorderWidget::sndExists(RecorderItem const *item) const {
	if (sndExistsCb == NULL)
		return NULL;
	return sndExistsCb(sndExistsCbData, item);
}

QSize RecorderWidget::sizeHint() const {
	return QSize(500, 500);
}

void RecorderWidget::resizeEvent(QResizeEvent *) {
	doLayoutAsync();
}

void RecorderWidget::doLayoutAsync() {
	disconnect(&timer, SIGNAL(timeout()), this, SLOT(doLayout()));
	connect(&timer, SIGNAL(timeout()), this, SLOT(doLayout()));
	timer.setSingleShot(true);
	timer.start(500);
}

void RecorderWidget::setFont(QFont font) {
	QWidget::setFont(font);
	doLayout();
}

void RecorderWidget::doLayout() {
	QRect r = rect();

	int x = r.x();
	int y = r.y();
	int spacing = 5;
	int lineHeight = 0;

	for (Items::iterator it = items.begin(); it != items.end(); it++) {
		RecorderItem *item = *it;

		int nextX = x + item->sizeHint().width() + spacing;
		if (nextX - spacing > r.right() && lineHeight > 0) {
			x = r.x();
			y = y + lineHeight + spacing;
			nextX = x + item->sizeHint().width() + spacing;
			lineHeight = 0;
		}

		item->setGeometry(QRect(QPoint(x, y), item->sizeHint()));

		x = nextX;
		lineHeight = qMax(lineHeight, item->sizeHint().height());

	}
	setMinimumHeight(y + lineHeight);
}

void RecorderWidget::addItem(RecorderItem *item, bool refresh) {
	items.append(item);
	if (refresh) doLayout();
}

void RecorderWidget::addItem(QString text, bool refresh) {
	RecorderItem *item = new RecorderItem(this, editor, text);
	addItem(item, refresh);
}

void RecorderWidget::addItem(RecorderItemData *data, bool refresh) {
	RecorderItem *item = new RecorderItem(this, editor, data);
	addItem(item, refresh);
}

bool RecorderWidget::validId(int id) {
	return (id >= 0 and id < items.count());
}

void RecorderWidget::scrollPage(RecorderItem *item) {
	QScrollBar *scrollBar = view->verticalScrollBar();
	int top = item->pos().ry();
	if (scrollBar->value() + view->height() < top + item->height()) scrollBar->setValue(top);
}

void RecorderWidget::setSelected(int id, bool paginate) {
	if (id == selected)
		return;

	if (validId(selected)) {
		items.at(selected)->setSelected(false);
	}

	selected = id;
	RecorderItem *item = validId(selected) ? items.at(selected) : NULL;

	if (item) {
		if (paginate) {
			scrollPage(item);
		}
		else {
			view->ensureWidgetVisible(item, 0, 0);
		}
		item->setSelected(true);

		emit selectionChanged(item);
	}
}

void RecorderWidget::setSelected(RecorderItem *item, bool paginate) {
	int id = items.indexOf(item);
	setSelected(id, paginate);
}

RecorderItem *RecorderWidget::itemSelected() {
	return !validId(selected)
		? NULL
		: items.at(selected);
}

bool RecorderWidget::move(int d, bool paginate) {
	if (!validId(selected) or !validId(selected + d))
		return false;

	setSelected(selected + d, paginate);
	return true;
}

int RecorderWidget::prevLine(int id) {
	if (!validId(id))
		return -1;

	QRect ref = items.at(id)->geometry();
	for (int i = id - 1; i >= 0; i--) {
		QRect rect = items.at(i)->geometry();
		if (rect.left() == 0 and rect.top() < ref.top())
			return i;
	}
	return -1;
}

int RecorderWidget::nextLine(int id) {
	if (!validId(id))
		return -1;

	QRect ref = items.at(id)->geometry();
	for (int i = id + 1; i < items.count(); i++) {
		QRect rect = items.at(i)->geometry();
		if (rect.top() > ref.top())
			return i;
	}
	return -1;
}

void RecorderWidget::del(int id) {
	if (!validId(id))
		return;

	RecorderItem *item = items.at(id);
	if (!item)
		return;

	emit deleteItem(item);

	delete item;
	items.remove(id);
	doLayoutAsync();

	if (selected == id) selected = -1;
	setSelected(validId(id) ? id : id - 1);
}

void RecorderWidget::edit(int id) {
	if (validId(id))
		items.at(id)->edit();
}

void RecorderWidget::insert(int) {
//	if (!validId(id))
//		return;

	RecorderItem *item = new RecorderItem(this, editor);
	addItem(item);
	setSelected(item);
	item->edit();
}

void RecorderWidget::keyPressEvent(QKeyEvent *event) {
        switch (event->key()) {
	case Qt::Key_Left:
		move(-1);
		break;
	case Qt::Key_Right:
		move(1);
		break;
	case Qt::Key_Up: {
		int id = prevLine(selected);
		if (id != -1) setSelected(id);
		break;
	}
	case Qt::Key_Down: {
		int id = nextLine(selected);
		if (id != -1) setSelected(id);
		break;
	}
	case Qt::Key_Insert:
		insert(selected);
		break;
	case Qt::Key_Delete:
		del(selected);
		break;
	case Qt::Key_Return:
		emit triggered();
		break;
	case Qt::Key_F2:
		edit(selected);
		break;
	default:
		QWidget::keyPressEvent(event);
		break;
	}
}

void RecorderWidget::import(QTextStream &stream) {
	for (QString line = stream.readLine(); !line.isNull(); line = stream.readLine()) {
		if (line != "")
			addItem(line, false);
	}
	doLayout();
}

void RecorderWidget::toVector(QVector<RecorderItemData *> &vector) const {
	for (Items::const_iterator it = items.begin(); it != items.end(); it++) {
		vector.append(new RecorderItemData(*it));
	}
}

void RecorderWidget::clear() {
	for (Items::iterator it = items.begin(); it != items.end(); it++) {
		RecorderItem *item = *it;
		delete item;
	}

	items.clear();
	setSelected(-1);
}

void RecorderWidget::fromVector(QVector<RecorderItemData *> &vector) {
	clear();
	QVector<RecorderItemData *>::const_iterator it;
	for (it = vector.begin(); it != vector.end(); it++){
		addItem(*it, false);
	}
	doLayout();
}

RecorderView::RecorderView(QWidget *parent, RecorderWidget::SndExistsCb *sndExistsCb, void *sndExistsCbData) : QScrollArea(parent) {
	widget = new RecorderWidget(this, sndExistsCb, sndExistsCbData);
	setWidgetResizable(true); 
	setWidget(widget);
}

int RecorderWidget::count() const {
	return items.count();
}

