#include "window_edit.h"

#include <QHeaderView>
#include <QApplication>
#include <QClipboard>
#include <QFile>
#include <QFileDialog>
#include <QFileSystemModel>
#include <QProcess>
#include <QMessageBox>
#include <QSettings>

WindowEdit::WindowEdit() {
	modelLanguages = new ModelItems;
	modelLanguages->add("", "");
	modelLanguages->parseXml(":xml/languages.xml");

	modelCountries = new ModelItems;
	modelCountries->add("", "");
	modelCountries->parseXml(":xml/countries.xml");

	statusBar = new QStatusBar(this);

	//tabs
	tabSpeaker = new TabSpeaker(this);
	tabCollection = new TabCollection(this);
	tabRecord = new TabRecord(this);

	tabWidget = new QTabWidget(this);
	tabWidget->addTab(tabSpeaker, QIcon(":/images/users.png"), tr("Speaker"));
	tabWidget->addTab(tabCollection, QIcon(":/images/collection.png"), tr("Collection"));
	tabWidget->addTab(tabRecord, QIcon(":/images/mic.png"), tr("Record"));
	tabWidget->setIconSize(QSize(24, 24));

	QAction *actionNew = new QAction(QIcon(":images/filenew.png"), tr("New"), this);
	connect(actionNew, SIGNAL(triggered()), this, SLOT(new_()));

	QAction *actionOpen = new QAction(QIcon(":images/fileopen.png"), tr("Open"), this);
	connect(actionOpen, SIGNAL(triggered()), this, SLOT(open()));

	QAction *actionSave = new QAction(QIcon(":images/filesave.png"), tr("Save"), this);
	connect(actionSave, SIGNAL(triggered()), this, SLOT(save()));

	QAction *actionSaveAs = new QAction(QIcon(":images/filesaveas.png"), tr("Save as"), this);
	connect(actionSaveAs, SIGNAL(triggered()), this, SLOT(saveAs()));

	QToolBar *toolbar = new QToolBar("Session");
	toolbar->addAction(actionNew);
	toolbar->addAction(actionOpen);
	toolbar->addAction(actionSave);
	toolbar->addAction(actionSaveAs);
	addToolBar(Qt::LeftToolBarArea, toolbar);

	setStatusBar(statusBar);
	setCentralWidget(tabWidget);
	resize(640, 480);
}

void WindowEdit::save(QString fileName) const {
	File file;
	toFile(file);
	file.write(fileName);
}

void WindowEdit::save() {
	if (currentFileName.isEmpty())
		saveAs();

	save(currentFileName);
}

void WindowEdit::saveAs() {
	QString fileName = QFileDialog::getSaveFileName(this, tr("Save session"), "", tr("Swac Record Session (*.sr)"));
	if (fileName.isNull())
		return;

	//Workarround. cf: QTBUG-27186
	QFileInfo info(fileName);
	if (info.suffix() == "")
		fileName += ".sr";

	currentFileName = fileName;
	save(fileName);
}

void WindowEdit::toTag(Tag &tag) const {
	tag.clear();
	tabSpeaker->toTag(tag);
	tabCollection->toTag(tag);
	tabRecord->toTag(tag);
}

void WindowEdit::toTag(Tag &tag, RecorderItem const *item) const {
	toTag(tag);
	item->toTag(tag);

	tag.add("SWAC_TECH_DATE", QDate::currentDate().toString("yyyy-MM-dd"));
	tag.add("SWAC_TECH_SOFT", "Swac Record");

	QString name = tag.get("SWAC_SPEAK_NAME");
	if (!name.isNull()) {
		tag.add("ARTIST", name);
	}

	QString lang = tag.get("SWAC_LANG");
	if (!lang.isNull()) {
		tag.add("TITLE", item->text() + " (" + lang + ")");
	}
}

void WindowEdit::toFile(File &file) const {
	toTag(file.tag);
	tabRecord->recorderView->widget->toVector(file.items);
}

void WindowEdit::fromFile(File &file) {
	tabSpeaker->fromTag(file.tag);
	tabCollection->fromTag(file.tag);
	tabRecord->fromTag(file.tag);
	tabRecord->recorderView->widget->fromVector(file.items);
}

void WindowEdit::open() {
	QString fileName = QFileDialog::getOpenFileName(this, tr("Open session"), "", tr("Swac Record Session (*.sr);;All files (*.*)"));
	if (fileName.isNull())
		return;

	currentFileName = fileName;
	File file;
	file.read(fileName);
	fromFile(file);
}

void WindowEdit::new_() {
	currentFileName = "";
	File file;
	fromFile(file);
}

TabSpeaker::TabSpeaker(WindowEdit *_parent) : QWidget(_parent), windowEdit(_parent) {
	name = new QLineEdit;

	nativeLang = new ItemsComboBox;
	nativeLang->setModel(windowEdit->modelLanguages);

	gender = new ItemsComboBox;
	gender->addItem(tr(""));
	gender->addItem(tr("Male"), "M");
	gender->addItem(tr("Female"), "F");

	yearOfBirth = new QLineEdit;
	yearOfBirth->setInputMask("0000");

	originCountry = new ItemsComboBox;
	originCountry->setModel(windowEdit->modelCountries);
	originRegion = new QLineEdit;

	livingTown = new QLineEdit;
	livingCountry = new ItemsComboBox;
	livingCountry->setModel(windowEdit->modelCountries);

	description = new QPlainTextEdit;

	QFormLayout *layout = new QFormLayout;
	layout->addRow(new QLabel(tr("Name:")), name);
	layout->addRow(new QLabel(tr("Native Language:")), nativeLang);
	layout->addRow(new QLabel(tr("Sex:")), gender);
	layout->addRow(new QLabel(tr("Year of Birth:")), yearOfBirth);
	layout->addRow(new QLabel(tr("Origin Country:")), originCountry);
	layout->addRow(new QLabel(tr("Origin Region:")), originRegion);
	layout->addRow(new QLabel(tr("Living Country:")), livingCountry);
	layout->addRow(new QLabel(tr("Living Town:")), livingTown);
	layout->addRow(new QLabel(tr("Description:")), description);

	setLayout(layout);
}

void TabSpeaker::toTag(Tag &tag) const {
	tag.add("SWAC_SPEAK_NAME", name->text());
	tag.add("SWAC_SPEAK_LANG", nativeLang->currentValue());
	tag.add("SWAC_SPEAK_GENDER", gender->currentValue());
	tag.add("SWAC_SPEAK_BIRTH_YEAR", yearOfBirth->text());
	tag.add("SWAC_SPEAK_LANG_COUNTRY", originCountry->currentValue());
	tag.add("SWAC_SPEAK_LANG_REGION", originRegion->text());
	tag.add("SWAC_SPEAK_LIV_COUNTRY", livingCountry->currentValue());
	tag.add("SWAC_SPEAK_LIV_TOWN", livingTown->text());
	tag.add("SWAC_SPEAK_DESC", description->toPlainText());
}

void TabSpeaker::fromTag(Tag &tag) {
	name->setText(tag.get("SWAC_SPEAK_NAME"));
	nativeLang->setValue(tag.get("SWAC_SPEAK_LANG"));
	gender->setValue(tag.get("SWAC_SPEAK_GENDER"));
	yearOfBirth->setText(tag.get("SWAC_SPEAK_BIRTH_YEAR"));
	originCountry->setValue(tag.get("SWAC_SPEAK_LANG_COUNTRY"));
	originRegion->setText(tag.get("SWAC_SPEAK_LANG_REGION"));
	livingCountry->setValue(tag.get("SWAC_SPEAK_LIV_COUNTRY"));
	livingTown->setText(tag.get("SWAC_SPEAK_LIV_TOWN"));
	description->setPlainText(tag.get("SWAC_SPEAK_DESC"));
}

TabCollection::TabCollection(WindowEdit *_parent) : QWidget(_parent), windowEdit(_parent) {
	name = new QLineEdit;
	description = new QPlainTextEdit;
	section = new QLineEdit;
	authors = new QLineEdit;
	license = new QLineEdit;
	copyright = new QLineEdit;
	directory = new QLineEdit;

	directorySelect = new QPushButton(tr("Select"));
	connect(directorySelect, SIGNAL(clicked()), this, SLOT(selectDirectory()));

	QHBoxLayout *directoryLayout = new QHBoxLayout;
	directoryLayout->addWidget(directory);
	directoryLayout->addWidget(directorySelect);

	QFormLayout *layout = new QFormLayout;
	layout->addRow(new QLabel(tr("Directory:")), directoryLayout);
	layout->addRow(new QLabel(tr("Name:")), name);
	layout->addRow(new QLabel(tr("Section:")), section);
	layout->addRow(new QLabel(tr("Authors:")), authors);
	layout->addRow(new QLabel(tr("License:")), license);
	layout->addRow(new QLabel(tr("Copyright:")), copyright);
	layout->addRow(new QLabel(tr("Description:")), description);

	connect(authors, SIGNAL(editingFinished()), this, SLOT(feedCopyright()));

	setLayout(layout);

	//Load settings
	QSettings settings;
	directory->setText(settings.value("directory", "").toString());
}

TabCollection::~TabCollection() {
	//Save settings
	QSettings settings;
	settings.setValue("directory", directory->text());
}

QString TabCollection::getCopyright(QString authors) {
	return QString::fromUtf8("Copyright© ") + QString::number(QDate::currentDate().year()) + " " + authors;
}

void TabCollection::feedCopyright() {
	if (copyright->text() == "") {
		copyright->setText(getCopyright(authors->text()));
	}
}

void TabCollection::selectDirectory() {
	QString path = QFileDialog::getExistingDirectory(this, tr("Collection Directory"));
	if (!path.isNull())
		directory->setText(path);
}

QString TabCollection::getDirectory() const {
	return directory->text();
}

void TabCollection::toTag(Tag &tag) const {
	tag.add("SWAC_COLL_NAME", name->text());
	tag.add("SWAC_COLL_SECTION", section->text());
	tag.add("SWAC_COLL_AUTHORS", authors->text());
	tag.add("SWAC_COLL_LICENSE", license->text());
	tag.add("SWAC_COLL_COPYRIGHT", copyright->text());
	tag.add("SWAC_COLL_DESC", description->toPlainText());
}

void TabCollection::fromTag(Tag &tag) {
	name->setText(tag.get("SWAC_COLL_NAME"));
	section->setText(tag.get("SWAC_COLL_SECTION"));
	authors->setText(tag.get("SWAC_COLL_AUTHORS"));
	license->setText(tag.get("SWAC_COLL_LICENSE"));
	copyright->setText(tag.get("SWAC_COLL_COPYRIGHT"));
	description->setPlainText(tag.get("SWAC_COLL_DESC"));
}

QAction *TabRecord::newAction(QIcon icon, QString toolTip, QObject *parent, QKeySequence shortcut) {
	QAction *action = new QAction(parent);
	action->setIcon(icon);
	action->setShortcut(shortcut);
	action->setToolTip(tr("%1 (%2)").arg(toolTip).arg(shortcut.toString()));
	return action;
}

TabRecord::TabRecord(WindowEdit *parent) : QWidget(parent), windowEdit(parent) {
	recorderView = new RecorderView(this, TabRecord::sndExistsCb, this);
	connect(recorderView->widget, SIGNAL(selectionChanged(RecorderItem *)), this, SLOT(selectedItemChanged(RecorderItem *)));
	connect(recorderView->widget, SIGNAL(deleteItem(RecorderItem *)), this, SLOT(deleteItem(RecorderItem *)));

	actionOpen = new QAction(QIcon(":images/add.png"), tr("Import from file"), this);
	connect(actionOpen, SIGNAL(triggered()), this, SLOT(open()));

	actionPaste = newAction(QIcon(":images/editpaste.png"), tr("Import from Clipboard"), this, QKeySequence::Paste);
	connect(actionPaste, SIGNAL(triggered()), this, SLOT(paste()));

	actionPlay = new QAction(QIcon(":images/hp.png"), tr("Play (Enter)"), this);
	connect(recorderView->widget, SIGNAL(triggered()), this, SLOT(play()));
	connect(actionPlay, SIGNAL(triggered()), this, SLOT(play()));

	actionDelete = newAction(QIcon(":images/button_cancel.png"), tr("Delete recording"), this, QKeySequence(Qt::CTRL + Qt::Key_D));
	connect(actionDelete, SIGNAL(triggered()), this, SLOT(deleteRecording()));

	actionRecord = newAction(QIcon(":images/record.png"), tr("Record"), this, QKeySequence(Qt::Key_Space));
	connect(actionRecord, SIGNAL(triggered()), this, SLOT(record()));

	actionPause = newAction(QIcon(":images/pause.png"), tr("Pause"), this, QKeySequence(Qt::Key_Space));
	connect(actionPause, SIGNAL(triggered()), this, SLOT(pause()));

	actionSettings = new QAction(QIcon(":images/settings.png"), tr("Settings"), this);
	connect(actionSettings, SIGNAL(triggered()), this, SLOT(setup()));

	lang = new ItemsComboBox;
	lang->setMaximumWidth(120);
	lang->setModel(windowEdit->modelLanguages);

	cutter = new CutterWidget(this);
	cutter->init();
	connect(cutter, SIGNAL(terminated()), this, SLOT(terminated()));
	connect(cutter, SIGNAL(changeState()), this, SLOT(updateRecordState()));
	updateRecordState();

	toolBar = new QToolBar;	
	toolBar->addWidget(lang);
	toolBar->addAction(actionOpen);
	toolBar->addAction(actionPaste);
	toolBar->addSeparator();
	toolBar->addAction(actionPlay);
	toolBar->addAction(actionDelete);
	toolBar->addAction(actionRecord);
	toolBar->addAction(actionPause);
	toolBar->addWidget(cutter);
	toolBar->addAction(actionSettings);

	layout = new QVBoxLayout;
	layout->addWidget(toolBar);
	layout->addWidget(recorderView);
	setLayout(layout);

	windowSetup = new WindowSetup(cutter, recorderView);

}

void TabRecord::setup() {
	windowSetup->exec();
}

void TabRecord::terminated() {
	if (!recorderView->widget->move(1, true))
		cutter->end();
}

QString TabRecord::sndFilename(QString const mask, RecorderItem const *item) const {
	QString str = mask;
	str.replace("$LANG", lang->currentValue());
	str.replace("$TEXT", item->text());
	str.replace("$EXT", "flac");
	return str;
}

QString TabRecord::sndPath(RecorderItem const *item) const {
	QString mask = !item->filename().isNull() ? item->filename() : "$LANG-$TEXT.$EXT";
	return QDir(windowEdit->tabCollection->getDirectory()).filePath(sndFilename(mask, item));
}

bool TabRecord::sndExists(RecorderItem const *item) const {
	QString path = sndPath(item);
	if (path.isNull())
		return false;

	return QFile::exists(path);
}

bool TabRecord::sndExistsCb(void *t, RecorderItem const *item) {
	return ((TabRecord *) t)->sndExists(item);
}

void TabRecord::setCutterOuput(RecorderItem const *item) {
	cutter->setTarget(sndPath(item));

	windowEdit->toTag(cutter->tag, item);
}

void TabRecord::updateRecordState() {
	bool state = cutter->isActive();
	recorderView->widget->setRecording(state);
	actionPause->setVisible(state);
	actionRecord->setVisible(!state);

	RecorderItem *item = recorderView->widget->itemSelected();
	actionPlay->setEnabled(item != NULL and item->sndExists());
	actionDelete->setEnabled(item != NULL and item->sndExists());
	actionRecord->setEnabled(item != NULL);

	if (item and state) setCutterOuput(item);
}

void TabRecord::selectedItemChanged(RecorderItem *) {
	updateRecordState();
}

void TabRecord::deleteItem(RecorderItem *item) {

	if (!item->sndExists())
		return;

	QMessageBox msgBox;
	msgBox.setText(tr("Do you want to delete associated sound file?"));
	msgBox.setStandardButtons(QMessageBox::Ok | QMessageBox::Cancel);
	msgBox.setDefaultButton(QMessageBox::Ok);

	if (msgBox.exec() == QMessageBox::Ok)
		deleteRecordingDo(item);
}

void TabRecord::deleteRecordingDo(RecorderItem *item) {
	QString path = sndPath(item);
	QFile::remove(path);
}

void TabRecord::deleteRecording() {
	RecorderItem *item = recorderView->widget->itemSelected();
	if (!item) return;
	deleteRecordingDo(item);
}

void TabRecord::play() {
	RecorderItem *item = recorderView->widget->itemSelected();
	if (!item or !item->sndExists()) return;

	QString path = sndPath(item);

	QFile::remove("output.wav");
	QStringList args;
	args << "-d" << path << "-o" << "output.wav";
	if (QProcess::execute("flac", args) == 0) {
		QStringList args2;
		args2 << "output.wav";
		QProcess::execute("aplay", args2);
		QFile::remove("output.wav");
	}
}

void TabRecord::record() {
	if (cutter->setup())
		cutter->begin();
}

void TabRecord::pause() {
	cutter->end();
}

void TabRecord::toTag(Tag &tag) const {
	tag.add("SWAC_LANG", lang->currentValue());
}

void TabRecord::fromTag(Tag &tag) {
	lang->setValue(tag.get("SWAC_LANG"));
}

void TabRecord::paste() {
	QString str = QApplication::clipboard()->text();
	QTextStream stream(&str);
	recorderView->widget->import(stream);
}

void TabRecord::open() {
	QString fileName = QFileDialog::getOpenFileName(this, tr("Import listing"), "", tr("Text Files (*.txt)"));
	if (fileName.isNull())
		return;

	QFile data(fileName);
	if (data.open(QFile::ReadOnly | QFile::Text)) {
		QTextStream stream(&data);
		recorderView->widget->import(stream);
	}
}

